
#include <stdarg.h>

void pr(char *fmt, ... ){
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.println(buf);

}

// Inputs
// AIN0 (pin 6) - potentiometer
// Analogue 1 - analogue in

// Outputs

// pins which drive the transmit transducer
// PWM on pins 3 and 11 (Timer 2)
#define OUT_PIN0 3
#define OUT_PIN1 11

 
#define AIN0 6
#define AIN1 7



#define BUFFER_SIZE 256


volatile unsigned int buffer[BUFFER_SIZE];
volatile unsigned int tail = 0;

volatile unsigned int icount;

#define ICLIMIT 50

int halfperiod=193;


ISR(TIMER1_CAPT_vect) 
{
 if(tail>=BUFFER_SIZE) 
 {
  icount++;
  if(icount>ICLIMIT)
  {
   TIMSK1 &= ~(1<<ICIE1); // stop capture interrupts
  }
  tail=0;
 }
 buffer[tail++] = ICR1;
}


void capture(int enable,int halfperiod)
{

 pinMode(AIN0, INPUT);
 pinMode(AIN1, INPUT);
 DIDR1  |=  (1<< AIN0D);         // disable digital input buffer AIN0/1
 DIDR1  |=  (1<< AIN1D);

 ADCSRA=0; // ADC Control and Status Register A
 ADCSRB=(1<<ACME); // enable muliplxer
 ADMUX=0; // ADC Multiplexer Selection Register
 ACSR=(1<<ACIC); // Analog Comparator Input Capture by Timer/Counter 1: On

 ADMUX=1;


 TCCR1A = 0;    // put timer1 in normal mode 
 TCCR1B = 0; // stop the clock
 TCNT1=0;
 
 TIFR1 = _BV(ICF1) | _BV(TOV1) | _BV(TOV2);  // clear any pending capture or overflow interrupts

 OCR1A=2*halfperiod-1;
 
 TCCR1B |= (1<< ICES1);   //switch edge detector to detect the rise
 
 // Enable input capture interrupt

 TCCR1B |= _BV(CS10) | _BV(WGM12); // start the clock, CTC mode, top in OCR1A

//http://www.openmusiclabs.com/learning/digital/synchronizing-timers/index.html

GTCCR = (1<<TSM)|(1<<PSRASY)|(1<<PSRSYNC); // halt all timers

// set all timers to the same value
TCNT0 = 0; // set timer0 to 0
TCNT1H = 0; // set timer1 high byte to 0
TCNT1L = 0; // set timer1 low byte to 0
TCNT2 = 0; // set timer2 to 0

GTCCR = 0; // release all timers

   tail=0;
 icount=0;

}



void pwm(int halfperiod)
{
  
 if(halfperiod==0)
 {
  TCCR2A = 0;    // put timer2 in normal mode 
  TCCR2B = 0; // stop the clock
  TCNT2=0;  
  return;
 }
  
 pinMode(OUT_PIN0,OUTPUT);
 pinMode(OUT_PIN1,OUTPUT);

 TCCR2A = 0;    // put timer1 in normal mode 
 TCCR2B = 0; // stop the clock
 TCNT2=0;
 TIMSK2=0;
 
 OCR2A=halfperiod-1;
 OCR2B=halfperiod-1;

//toggle OC1A pin as suggested by https://www.avrfreaks.net/comment/748887#comment-748887 
 TCCR2A = _BV(COM2A1) | _BV(COM2B1) | _BV(COM2B0);
 TCCR2B = _BV(FOC2A) | _BV(FOC2B);

 TCCR2A = _BV(COM2A0) | _BV(COM2B0) | _BV(WGM21); // set to toggle OC1A/OC1B on match

 TCCR2B = _BV(CS20); // start the clock, CTC mode, top in OCR1A
}


 

void loop() 
{
 TIMSK0 &= ~_BV(TOIE0); // disable timer0 overflow interrupt

 icount=0;
 TIMSK1 |= (1<<ICIE1); // enable capture interrupts
 while(icount<=ICLIMIT);

 TIMSK0 |= _BV(TOIE0); // enable timer0 overflow interrupt


 unsigned long sum=0;
 for(int i=0;i<BUFFER_SIZE;i+=1) sum+=buffer[i];
 int average=sum/BUFFER_SIZE;
 int remainder=sum-average*BUFFER_SIZE;
 remainder*=10;
 remainder/=256;
 pr("%d.%d",average,remainder);
}



void setup() 
{
 Serial.begin(9600);
 while (!Serial) { ; /* wait for serial port to connect. Needed for native USB */}

 pwm(halfperiod);
 capture(1,halfperiod);
}


